##############################################
# cube1
cube1 <- NULL
cube1$data <- array(rnorm(10), dim = c(lat = 2, time = 5))
class(cube1) <- 's2dv_cube'

# dat1
dims <- c(syear = 6, time = 3, latitude = 2, longitude = 1, ensemble = 10)
dimscor <- c(syear = 1, time = 3, latitude = 2, longitude = 1, ensemble = 15)

set.seed(1)
exp_tasmax <- array(rnorm(360, 27.73, 5.26), dim = dims)
set.seed(2)
exp_tasmin <- array(rnorm(360, 14.83, 3.86), dim = dims)
set.seed(3)
exp_prlr <- array(rnorm(360, 21.19, 25.64), dim = dims)

set.seed(1)
expcor_tasmax <- array(rnorm(60, 29.03, 5.67), dim = dimscor)
set.seed(2)
expcor_tasmin <- array(rnorm(60, 15.70, 4.40), dim = dimscor)
set.seed(3)
expcor_prlr <- array(rnorm(60, 15.62, 21.38), dim = dimscor)

dates_exp <- as.Date(c(paste0(2010:2015, "-08-16"), paste0(2010:2015, "-09-15"), 
                   paste0(2010:2015, "-10-16")))
dim(dates_exp) <- c(sday = 1, sweek = 1, syear = 6, time = 3)

dates_expcor <- as.Date(c(paste0(2020, "-08-16"), paste0(2020, "-09-15"), 
                   paste0(2020, "-10-16")))
dim(dates_expcor) <- c(sday = 1, sweek = 1, syear = 1, time = 3)

lat <- c(40,40.1)

exp1 <- list('tasmax' = exp_tasmax, 'tasmin' = exp_tasmin, 'prlr' = exp_prlr)
exp_cor1 <- list('tasmax' = expcor_tasmax, 'tasmin' = expcor_tasmin, 'prlr' = expcor_prlr)

res <- PeriodSPEI(exp = exp1, exp_cor = exp_cor1, lat = lat, 
                  dates_exp = dates_exp, dates_expcor = dates_expcor)

##############################################

test_that("1. Initial checks CST_PeriodSPEI", {
  expect_error(
    CST_PeriodSPEI(exp = NULL), 
    "Parameter 'exp' cannot be NULL."
  )
  expect_error(
    CST_PeriodSPEI(exp = array(10)), 
    "Parameter 'exp' must be a list of 's2dv_cube' class."
  )
  expect_error(
    CST_PeriodSPEI(exp = list(cube1)), 
    paste0("Spatial coordinate names of parameter 'exp' do not match any ",
           "of the names accepted by the package.")
  )
})

##############################################

test_that("1. Initial checks PeriodSPEI", {
  # exp
  expect_error(
    PeriodSPEI(exp = NULL), 
    "Parameter 'exp' needs to be a named list with the needed variables."
  )
  expect_error(
    PeriodSPEI(exp = list(1)), 
    "Parameter 'exp' needs to be a named list with the variable names."
  )
  expect_error(
    PeriodSPEI(exp = list('tasmax' = array(10))), 
    "Parameter 'exp' needs to be a list of arrays with dimension names."
  )
  # exp_cor
  expect_error(
    PeriodSPEI(exp = exp1, exp_cor = 1), 
    paste0("Parameter 'exp_cor' needs to be a named list with the needed ", 
           "variables if it is not NULL.")
  )
  expect_error(
    PeriodSPEI(exp = exp1, exp_cor = list(1)), 
    "Parameter 'exp_cor' needs to be a named list with the variable names."
  )
  expect_error(
    PeriodSPEI(exp = exp1, exp_cor = list('tasmax' = array(10))), 
    "Parameter 'exp_cor' needs to be a list of arrays with dimension names."
  )
  expect_error(
    PeriodSPEI(exp = list('tas' = array(10, dim = c(time = 10))), 
               exp_cor = list('tos' = array(10, dim = c(time = 10)))), 
    "Parameter 'exp_cor' needs to have the same variable names as 'exp'."
  )
  # exp (2)
  expect_warning(
    PeriodSPEI(exp = exp1, pet_method = '1', dates_exp = dates_exp, lat = lat), 
    paste0("Parameter 'pet_method' needs to be 'hargreaves' or ", 
           "'hargreaves_modified'. It is set to 'hargreaves'.")
  )
  # time_dim
  expect_error(
    PeriodSPEI(exp = exp1, time_dim = 1, dates_exp = dates_exp, lat = lat)
  )
  # leadtime_dim
  expect_error(
    PeriodSPEI(exp = exp1, time_dim = 1, dates_exp = dates_exp, lat = lat)
  )
  # memb_dim
  expect_error(
    PeriodSPEI(exp = exp1, memb_dim = 1, dates_exp = dates_exp, lat = lat)
  )
  # lat_dim
  expect_error(
    PeriodSPEI(exp = exp1, lat_dim = 1, dates_exp = dates_exp, lat = lat)
  )
  # accum
  expect_error(
    PeriodSPEI(exp = exp1, accum = 10, dates_exp = dates_exp, lat = lat)
  ) 

})

##############################################

test_that("2. Output checks", {
  res <- PeriodSPEI(exp = exp1, exp_cor = exp_cor1, lat = lat, 
                    dates_exp = dates_exp, dates_expcor = dates_expcor)
  expect_equal(
    length(res), 
    3
  )
  expect_equal(
    dim(res[[1]]),
    dims
  )
  expect_equal(
    dim(res[[3]])[which(!names(dim(res[[3]])) %in% c('coef', 'syear'))],
    dims[which(!names(dims) %in% c('syear', 'ensemble'))]
  )
})

##############################################